@@ -16,7 +16,6 @@
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
-#ident "$Id: vtepango.c,v 1.16 2004/04/20 05:16:56 nalin Exp $"
 
 #include "../config.h"
 
@@ -55,7 +54,7 @@ _vte_pango_create(struct _vte_draw *draw, GtkWidget *widget)
 {
 	struct _vte_pango_data *data;
 
-	draw->impl_data = g_malloc(sizeof(struct _vte_pango_data));
+	draw->impl_data = g_slice_new(struct _vte_pango_data);
 	data = (struct _vte_pango_data*) draw->impl_data;
 
 	data->color.red = 0;
@@ -75,33 +74,20 @@ _vte_pango_destroy(struct _vte_draw *draw)
 	struct _vte_pango_data *data;
 	data = (struct _vte_pango_data*) draw->impl_data;
 
-	data->scrollx = data->scrolly = 0;
-
-	if (GDK_IS_PIXMAP(data->pixmap)) {
-		g_object_unref(G_OBJECT(data->pixmap));
-		data->pixmap = NULL;
-		data->pixmapw = data->pixmaph = 0;
+	if (data->pixmap != NULL) {
+		g_object_unref(data->pixmap);
 	}
 	if (data->font != NULL) {
 		pango_font_description_free(data->font);
-		data->font = NULL;
 	}
-	if (PANGO_IS_LAYOUT(data->layout)) {
-		g_object_unref(G_OBJECT(data->layout));
-		data->layout = NULL;
+	if (data->layout != NULL) {
+		g_object_unref(data->layout);
 	}
-	if (PANGO_IS_CONTEXT(data->ctx)) {
-		g_object_unref(G_OBJECT(data->ctx));
-		data->ctx = NULL;
-	}
-	if (GDK_IS_GC(data->gc)) {
-		g_object_unref(G_OBJECT(data->gc));
-		data->gc = NULL;
+	if (data->gc != NULL) {
+		g_object_unref(data->gc);
 	}
 
-	memset(&data->color, 0, sizeof(data->color));
-
-	g_free(draw->impl_data);
+	g_slice_free(struct _vte_pango_data, draw->impl_data);
 }
 
 static GdkVisual *
@@ -119,34 +105,22 @@ _vte_pango_get_colormap(struct _vte_draw *draw)
 static void
 _vte_pango_start(struct _vte_draw *draw)
 {
-#if GTK_CHECK_VERSION(2,2,0)
-	GdkScreen *screen;
-#endif
 	PangoContext *ctx;
 	struct _vte_pango_data *data;
 	data = (struct _vte_pango_data*) draw->impl_data;
 
-#if GTK_CHECK_VERSION(2,2,0)
-	screen = gdk_drawable_get_screen(draw->widget->window);
-	ctx = gdk_pango_context_get_for_screen(screen);
-#else
-	ctx = gdk_pango_context_get();
-#endif
-	if (PANGO_IS_CONTEXT(data->ctx)) {
-		g_object_unref(G_OBJECT(data->ctx));
-	}
-	data->ctx = ctx;
+	ctx = gtk_widget_get_pango_context (draw->widget);
 
-	if (PANGO_IS_LAYOUT(data->layout)) {
-		g_object_unref(G_OBJECT(data->layout));
+	if (data->layout != NULL) {
+		g_object_unref(data->layout);
 	}
-	data->layout = pango_layout_new(data->ctx);
-	if (data->font) {
+	data->layout = pango_layout_new(ctx);
+	if (data->font != NULL) {
 		pango_layout_set_font_description(data->layout, data->font);
 	}
 
-	if (GDK_IS_GC(data->gc)) {
-		g_object_unref(G_OBJECT(data->gc));
+	if (data->gc != NULL) {
+		g_object_unref(data->gc);
 	}
 	data->gc = gdk_gc_new(draw->widget->window);
 
@@ -160,24 +134,19 @@ _vte_pango_end(struct _vte_draw *draw)
 	struct _vte_pango_data *data;
 	data = (struct _vte_pango_data*) draw->impl_data;
 
-	if (GDK_IS_GC(data->gc)) {
-		g_object_unref(G_OBJECT(data->gc));
+	if (data->gc != NULL) {
+		g_object_unref(data->gc);
 	}
 	data->gc = NULL;
 
-	if (PANGO_IS_LAYOUT(data->layout)) {
-		g_object_unref(G_OBJECT(data->layout));
+	if (data->layout != NULL) {
+		g_object_unref(data->layout);
 	}
 	data->layout = NULL;
-
-	if (PANGO_IS_CONTEXT(data->ctx)) {
-		g_object_unref(G_OBJECT(data->ctx));
-	}
-	data->ctx = NULL;
 }
 
 static void
-_vte_pango_set_background_color(struct _vte_draw *draw, GdkColor *color)
+_vte_pango_set_background_color(struct _vte_draw *draw, GdkColor *color, guint16 opacity)
 {
 	struct _vte_pango_data *data;
 	data = (struct _vte_pango_data*) draw->impl_data;
@@ -194,13 +163,17 @@ _vte_pango_set_background_image(struct _vte_draw *draw,
 {
 	GdkPixmap *pixmap;
 	struct _vte_pango_data *data;
+	GdkScreen *screen;
+
+	screen = gtk_widget_get_screen(draw->widget);
 
 	data = (struct _vte_pango_data*) draw->impl_data;
-	pixmap = vte_bg_get_pixmap(vte_bg_get(), type, pixbuf, file,
+	pixmap = vte_bg_get_pixmap(vte_bg_get_for_screen(screen),
+				   type, pixbuf, file,
 				   color, saturation,
 				   _vte_draw_get_colormap(draw, TRUE));
-	if (GDK_IS_PIXMAP(data->pixmap)) {
-		g_object_unref(G_OBJECT(data->pixmap));
+	if (data->pixmap != NULL) {
+		g_object_unref(data->pixmap);
 	}
 	data->pixmap = NULL;
 	data->pixmapw = data->pixmaph = 0;
@@ -210,6 +183,13 @@ _vte_pango_set_background_image(struct _vte_draw *draw,
 	}
 }
 
+static void
+_vte_pango_clip(struct _vte_draw *draw, GdkRegion *region)
+{
+	struct _vte_pango_data *data = draw->impl_data;
+	gdk_gc_set_clip_region(data->gc, region);
+}
+
 static void
 _vte_pango_clear(struct _vte_draw *draw,
 		 gint x, gint y, gint width, gint height)
@@ -261,31 +241,19 @@ _vte_pango_set_text_font(struct _vte_draw *draw,
 			 const PangoFontDescription *fontdesc,
 			 VteTerminalAntiAlias antialias)
 {
-#if GTK_CHECK_VERSION(2,2,0)
-	GdkScreen *screen;
-#endif
 	PangoContext *ctx;
 	PangoLayout *layout;
 	PangoLayoutIter *iter;
 	PangoRectangle ink, logical;
-	gunichar full_codepoints[] = {VTE_DRAW_DOUBLE_WIDE_CHARACTERS};
+	gunichar full_codepoints[] = {VTE_DRAW_DOUBLE_WIDE_IDEOGRAPHS};
 	GString *full_string;
 	gint full_width;
-	int i;
+	guint i;
 	struct _vte_pango_data *data;
 
 	data = (struct _vte_pango_data*) draw->impl_data;
 
-#if GTK_CHECK_VERSION(2,2,0)
-	if (gtk_widget_has_screen(draw->widget)) {
-		screen = gtk_widget_get_screen(draw->widget);
-	} else {
-		screen = gdk_display_get_default_screen(gtk_widget_get_display(draw->widget));
-	}
-	ctx = gdk_pango_context_get_for_screen(screen);
-#else
-	ctx = gdk_pango_context_get();
-#endif
+	ctx = gtk_widget_get_pango_context(draw->widget);
 	layout = pango_layout_new(ctx);
 	if (data->font != NULL) {
 		pango_font_description_free(data->font);
@@ -308,7 +276,7 @@ _vte_pango_set_text_font(struct _vte_draw *draw,
 
 	/* Estimate for CJK characters. */
 	full_width = draw->width * 2;
-	full_string = g_string_new("");
+	full_string = g_string_new(NULL);
 	for (i = 0; i < G_N_ELEMENTS(full_codepoints); i++) {
 		g_string_append_unichar(full_string, full_codepoints[i]);
 	}
@@ -319,7 +287,8 @@ _vte_pango_set_text_font(struct _vte_draw *draw,
 
 	/* If they're the same, then we have a screwy font. */
 	if (full_width == draw->width) {
-		draw->width /= 2;
+		/* add 1 to round up when dividing by 2 */
+		draw->width = (draw->width + 1) / 2;
 	}
 
 	draw->width = PANGO_PIXELS(draw->width);
@@ -332,15 +301,11 @@ _vte_pango_set_text_font(struct _vte_draw *draw,
 	}
 	pango_layout_iter_free(iter);
 
-#ifdef VTE_DEBUG
-	if (_vte_debug_on(VTE_DEBUG_MISC)) {
-		fprintf(stderr, "VtePango font metrics = %dx%d (%d).\n",
+	_vte_debug_print(VTE_DEBUG_MISC,
+			"VtePango font metrics = %dx%d (%d).\n",
 			draw->width, draw->height, draw->ascent);
-	}
-#endif
 
-	g_object_unref(G_OBJECT(layout));
-	g_object_unref(G_OBJECT(ctx));
+	g_object_unref(layout);
 }
 
 static int
@@ -383,7 +348,7 @@ _vte_pango_draw_text(struct _vte_draw *draw,
 {
 	struct _vte_pango_data *data;
 	char buf[VTE_UTF8_BPC];
-	int i;
+	guint i;
 	gsize length;
 	GdkColor wcolor;
 
@@ -412,6 +377,12 @@ _vte_pango_draw_char(struct _vte_draw *draw,
 	return TRUE;
 }
 
+static gboolean
+_vte_pango_draw_has_char(struct _vte_draw *draw, gunichar c)
+{
+	return FALSE;
+}
+
 static void
 _vte_pango_draw_rectangle(struct _vte_draw *draw,
 			  gint x, gint y, gint width, gint height,
@@ -457,8 +428,8 @@ _vte_pango_set_scroll(struct _vte_draw *draw, gint x, gint y)
 	data->scrolly = y;
 }
 
-struct _vte_draw_impl _vte_draw_pango = {
-	"VtePango", "VTE_USE_PANGO",
+const struct _vte_draw_impl _vte_draw_pango = {
+	"Pango",
 	_vte_pango_check,
 	_vte_pango_create,
 	_vte_pango_destroy,
@@ -469,6 +440,7 @@ struct _vte_draw_impl _vte_draw_pango = {
 	_vte_pango_set_background_color,
 	_vte_pango_set_background_image,
 	FALSE,
+	_vte_pango_clip,
 	_vte_pango_clear,
 	_vte_pango_set_text_font,
 	_vte_pango_get_text_width,
@@ -478,6 +450,7 @@ struct _vte_draw_impl _vte_draw_pango = {
 	_vte_pango_get_using_fontconfig,
 	_vte_pango_draw_text,
 	_vte_pango_draw_char,
+	_vte_pango_draw_has_char,
 	_vte_pango_draw_rectangle,
 	_vte_pango_fill_rectangle,
 	_vte_pango_set_scroll,
